@@ -15,7 +15,7 @@ from flipper.kernel.decorators import memoize  # Special import needed for decor
 INFTY = float('inf')
 HASH_DENOMINATOR = 30
 
-class Lamination(object):
+class Lamination:
     ''' This represents a lamination on an triangulation.
     
     It is given by a list of its geometric intersection numbers and a
@@ -80,8 +80,6 @@ class Lamination(object):
         return self.triangulation == other.triangulation and \
             all(v == w for v, w in zip(self.geometric, other.geometric)) and \
             all(v == w for v, w in zip(self.algebraic, other.algebraic))
-    def __ne__(self, other):
-        return not self == other
     def __hash__(self):
         # This should be done better.
         return hash(tuple(self.geometric) + tuple(self.algebraic))
